@@ -132,6 +132,11 @@ class CastOp(UnaryUnaryOp):#{{{
 class DetectTypeOp(UnaryUnaryOp):#{{{
     __slots__ = []#}}}
 
+
+class DetectFixedShapesOp(UnaryUnaryOp):
+    __slots__ = []
+
+
 class UnpackArrayOp(UnaryUnaryOp):#{{{
     """An slice which is the result of unpacking a source slice."""
     __slots__ = ["unpack_dims"]
@@ -219,7 +224,7 @@ class BroadcastOp(UnaryUnaryOp):#{{{
                 assert refslices[bcpos],"No refslices for broadcast of dim: " + str(bcdims[pos])
                 bcpos += 1
                 ndims, ntype = ndims.updateDim(pos, bcdims[pos], ntype)
-            elif(planelem == BCCOPY):
+            elif(planelem == BCCOPY or planelem == BCSOURCE):
                 pass
             else:
                 raise RuntimeError, "Unknown plan element in plan: " + str(plan)
@@ -227,6 +232,8 @@ class BroadcastOp(UnaryUnaryOp):#{{{
         UnaryUnaryOp.__init__(self, slice, dims=ndims,rtype=ntype)#}}}
 
 def broadcast(slices,mode="pos", partial=False):#{{{
+    if not slices:
+        return ((),())
     slicedimpaths = [s.dims for s in slices]
     if(mode == "dim"):
         bcdims, bcplan = dimpaths.planBroadcastMatchDim(slicedimpaths)
@@ -243,60 +250,75 @@ def broadcast(slices,mode="pos", partial=False):#{{{
 
     nslices = []
     for plan,slice in zip(bcplan,slices):
-        nplan = []
-        active_bcdims = []
-        for dimpos, bcdim, planelem in zip(range(len(bcdims)),bcdims,plan):
-            if(planelem == BCNEW):
-                ndim = dimensions.Dim(1)
-                slice = InsertDimOp(slice,dimpos,ndim)
-                active_bcdims.append(bcdim)
-                nplan.append(BCEXIST)
-            elif(planelem == BCENSURE):
-                slice = EnsureCommonDimOp(slice,references[bcdim],dimpos,bcdim)
-                nplan.append(BCCOPY)
-            elif(planelem == BCEXIST):
-                active_bcdims.append(bcdim)
-                nplan.append(planelem)
-            elif(planelem == BCINSERT):
-                ndim = dimensions.Dim(1)
-                slice = InsertDimOp(slice,dimpos,ndim)
-                nplan.append(BCCOPY)
-            else:
-                nplan.append(planelem)
-        if(active_bcdims):                
-            slice = BroadcastOp(slice,[references[bcdim] for bcdim in active_bcdims],nplan,bcdims,partial)
+        slice = apply_broadcast_plan(slice, plan, bcdims, references, partial)
         nslices.append(slice)
     return (nslices, bcplan)
     #}}}
 
-def apply_broadcast_plan(slices,bcdims, bcplan):#{{{
+def broadcast_fromplan(slice, refslices, plan, origdims, bcdims,  partial=False):#{{{
     references = defaultdict(list)
     for bcdim in bcdims:
-        for slice in slices:
-            if bcdim in slice.dims:
-                references[bcdim].append(slice)
-   
-    nslices = []
-    for plan,slice in zip(bcplan,slices):
-        nplan = []
-        active_bcdims = []
-        for dimpos, bcdim, planelem in zip(range(len(bcdims)),bcdims,plan):
-            if(planelem == BCNEW):
-                ndim = dimensions.Dim(1)
-                slice = InsertDimOp(slice,dimpos,ndim)
-                active_bcdims.append(bcdim)
-                nplan.append(BCEXIST)
-            elif(planelem == BCENSURE):
-                slice = EnsureCommonDimOp(slice,references[bcdim],dimpos,bcdim)
-                nplan.append(BCCOPY)
-            elif(planelem == BCEXIST):
-                active_bcdims.append(bcdim)
-                nplan.append(planelem)
-            else:
-                nplan.append(planelem)
-        if(active_bcdims):                
-            slice = BroadcastOp(slice,[references[bcdim] for bcdim in active_bcdims],nplan,bcdims)
-        nslices.append(slice)#}}}
+        for rslice in refslices:
+            if bcdim in rslice.dims:
+                references[bcdim].append(rslice)
+
+    sbcdims,splan = dimpaths.planBroadcastFromPlan(slice.dims, plan, origdims, bcdims)
+    slice = apply_broadcast_plan(slice, splan, sbcdims, references, partial)
+    return (slice, splan)
+    #}}}
+
+def apply_broadcast_plan(slice,plan, bcdims, references, partial):#{{{
+    nplan = []
+    active_bcdims = []
+    for dimpos, bcdim, planelem in zip(range(len(bcdims)),bcdims, plan):
+        if(planelem == BCNEW):
+            ndim = dimensions.Dim(1)
+            slice = InsertDimOp(slice,dimpos,ndim)
+            active_bcdims.append(bcdim)
+            nplan.append(BCEXIST)
+        elif(planelem == BCENSURE):
+            slice = EnsureCommonDimOp(slice,references[bcdim],dimpos,bcdim)
+            nplan.append(BCSOURCE)
+        elif(planelem == BCEXIST):
+            active_bcdims.append(bcdim)
+            nplan.append(planelem)
+        else:
+            nplan.append(planelem)
+    if(active_bcdims):                
+        slice = BroadcastOp(slice,[references[bcdim] for bcdim in active_bcdims], nplan, bcdims, partial)
+    return slice
+#}}}
+
+def broadcastParentsFromPlan(slice, seldimpath, plan, origdims, bcdims, refslices, partial):#{{{
+    used_dims = [False,] * len(slice.dims)
+    if not plan:
+        return slice
+    while True:
+        #determine filter dim
+        lastpos = slice.dims.matchDimPath(seldimpath)
+        for filterpos in lastpos[::-1]:
+             if(not used_dims[filterpos]):
+                break
+        else:
+            break
+
+        #pack up to and including filter dim
+        packdepth = len(slice.dims) - filterpos
+        if(packdepth):
+            slice = PackArrayOp(slice,packdepth)
+
+        #broadcast to constraint
+        slice,splan = broadcast_fromplan(slice, refslices, plan, origdims, bcdims, partial)
+
+        #adapt used_dims
+        used_dims = dimpaths.applyPlan(used_dims,splan,newvalue=True,copyvalue=True,ensurevalue=True,existvalue=True)
+        used_dims[filterpos] = True
+
+        #unpack dims
+        if(packdepth):
+            slice = UnpackArrayOp(slice,packdepth)
+            
+    return slice#}}}
 
 class PermuteDimsOp(UnaryUnaryOp):#{{{
     __slots__ = ["permute_idxs"]
@@ -333,32 +355,30 @@ class FilterOp(UnaryUnaryOp):#{{{
     def __init__(self,slice,constraint, ndim):
         stype = slice.type
         assert isinstance(stype, rtypes.TypeArray), "Filter on non-array type not possible"
-        sdims = stype.dims
-        if(ndim is None):
-            sdims, subtype = sdims.removeDim(0, constraint, stype.subtypes[0])
-        else:
-            sdims, subtype = sdims.updateDim(0, ndim, stype.subtypes[0])
         
         if(isinstance(constraint.type,rtypes.TypeArray)):
             has_missing = constraint.type.subtypes[0].has_missing
         else:
             has_missing = constraint.type.has_missing
         self.has_missing = has_missing            
-        
-        array_has_missing = stype.has_missing
-        if(has_missing):
-            if(isinstance(constraint.type,rtypes.TypeSlice)):
-                array_has_missing = True 
-            else:
-                if not subtype.has_missing:
-                    subtype = subtype.copy()
-                    subtype.has_missing = True
 
+        sdims = stype.dims
+        if(ndim is None):
+            sdims, subtype = sdims.removeDim(0, constraint, stype.subtypes[0])
+        else:
+            sdims, subtype = sdims.updateDim(0, ndim, stype.subtypes[0])
+        
+        
         if(sdims):
             ntype = dimpaths.dimsToArrays(sdims, subtype)
         else:
             ntype = subtype
         
+        if(has_missing):
+            ntype = ntype.setHasMissing(True)
+            if not subtype.hasMissingValInfo():
+                slice = DetectFixedShapesOp(slice)
+      
         self.constraint = constraint
         UnaryUnaryOp.__init__(self, slice, rtype=ntype)#}}}
 
@@ -377,6 +397,8 @@ def filter(slice,constraint,seldimpath, ndim, mode="dim"):#{{{
         packdepth = len(slice.dims) - filterpos
         if(packdepth):
             slice = PackArrayOp(slice,packdepth)
+
+
         #prepare adaptation of ndim.dependent
         if(not ndim is None):
             dep = list(ndim.dependent)
@@ -384,21 +406,23 @@ def filter(slice,constraint,seldimpath, ndim, mode="dim"):#{{{
                 dep.insert(0,False)
 
         #broadcast to constraint
-        (slice,constraint),(splan,cplan) = broadcast([slice,constraint],mode=mode, partial=True)
+        (slice,tconstraint),(splan,cplan) = broadcast([slice,constraint],mode=mode, partial=True)
 
         #adapt ndim to braodcast, apply filter
+        #FIXME: change dependent dims of slice.type
+
         if(not ndim is None):
-            if(isinstance(constraint.type,rtypes.TypeSlice)):
+            if(isinstance(tconstraint.type,rtypes.TypeSlice)):
                 ndep = dimpaths.applyPlan(dep,cplan,newvalue=True, copyvalue=True, ensurevalue=True)
             else:
                 ndep = dimpaths.applyPlan(dep,cplan,newvalue=False)
             xndim = ndim.changeDependent(tuple(ndep), slice.dims)
         else:
             xndim = ndim
-        slice = FilterOp(slice,constraint,xndim)
+        slice = FilterOp(slice,tconstraint,xndim)
 
         #adapt used_dims
-        used_dims = dimpaths.applyPlan(used_dims,splan,newvalue=True,copyvalue=True,ensurevalue=True)
+        used_dims = dimpaths.applyPlan(used_dims,splan,newvalue=True,copyvalue=True,ensurevalue=True, existvalue=True)
         
         #handle dim removal, filter_dim is used
         if(ndim is None): #dimension removed/collapsed
@@ -497,7 +521,6 @@ class NoneToMissingOp(UnaryUnaryOp):#{{{
         assert slice.type.has_missing, "Cannot apply none to missing on type without has_missing"
         UnaryUnaryOp.__init__(self, slice)#}}}
     
-
 class FreezeOp(UnaryUnaryOp):#{{{
     __slots__ = []
 #}}}
@@ -588,13 +611,46 @@ class PackTupleOp(MultiUnaryOp):#{{{
         nbookmarks = reduce(set.union,[slice.bookmarks for slice in slices])
         MultiUnaryOp.__init__(self, slices, name=field, rtype=ntype, dims=iter(cdim).next(),bookmarks=nbookmarks)#}}}
 
-class PackDictOp(PackTupleOp):
+class PackDictOp(PackTupleOp):#{{{
     __slots__ = ["with_missing"]
     ocls = rtypes.TypeRecordDict
 
     def __init__(self, slices, field="data", with_missing=False):
         self.with_missing = with_missing
-        PackTupleOp.__init__(self, slices, field)
+        PackTupleOp.__init__(self, slices, field)#}}}
+
+
+class PackIndexDictOp(MultiUnaryOp):#{{{
+    __slots__ = []
+    ocls = rtypes.TypeIndexDict
+
+    def __init__(self, slices, field="data"):
+        assert len(slices) == 2, "Index dict op should have two slices"
+        assert slices[0].dims == slices[1].dims, "Key and value of dict should have same dimension"
+
+        slices = [PackArrayOp(slice) for slice in slices]
+        ndims = dimpaths.DimPath(dimensions.Dim(UNDEFINED, (), name="dict"))
+        subtypes = [dimpaths.dimsToArrays(ndims, slice.type.subtypes[0]) for slice in slices]
+        
+        fieldnames = [slice.name for slice in slices]
+        ntype = self.ocls(False, tuple(subtypes), tuple(fieldnames))
+        nbookmarks = reduce(set.union,[slice.bookmarks for slice in slices])
+        MultiUnaryOp.__init__(self, slices, name=field, rtype=ntype, dims=slices[0].dims, bookmarks=nbookmarks)#}}}
+        #}}}
+
+
+class TakeOp(MultiUnaryOp):
+    __slots__ = ['allow_missing']
+    def __init__(self, source_slice, take_slice, allow_missing=False):
+        assert isinstance(source_slice.type, rtypes.TypeIndexDict), "Take left op should have type index dict"
+        source_slice, take_slice = broadcast((source_slice, take_slice), mode="dim")[0]
+        ntype = source_slice.type.subtypes[1].subtypes[0]
+        if allow_missing:
+            ntype = ntype.setHasMissing(True)
+        name = source_slice.type.fieldnames[1]
+        nbookmarks = source_slice.bookmarks | take_slice.bookmarks
+        self.allow_missing = allow_missing
+        MultiUnaryOp.__init__(self, [source_slice, take_slice], name=name, rtype=ntype, dims=take_slice.dims, bookmarks = nbookmarks)#}}}
 
 class GroupIndexOp(MultiUnaryOp):#{{{
     def __init__(self, slices):
@@ -647,37 +703,29 @@ class MultiOp(Op):
         assert isinstance(results, tuple), "Results should be a tuple"
         self.results = results
 
-class SelectOp(UnaryUnaryOp):
+class SelectOp(UnaryUnaryOp):#{{{
     __slots__ = ["index"]
     def __init__(self, slice, index, name, rtype, ndims, bookmarks):
         assert isinstance(slice, MultiOp), "Source lice of SelectOp should be a multiop"
         self.index = index
-        UnaryUnaryOp.__init__(self, slice, name=name, rtype=rtype, dims=ndims, bookmarks=bookmarks)
+        UnaryUnaryOp.__init__(self, slice, name=name, rtype=rtype, dims=ndims, bookmarks=bookmarks)#}}}
 
 class UnaryMultiOp(MultiOp):
     __slots__ = ["source"]
 
-class MultiMultiOp(MultiOp):
+class MultiMultiOp(MultiOp):#{{{
     __slots__ = ["sources"]
 
     def __init__(self, sources, results):
         assert isinstance(sources, tuple), "Sources should be a tuple"
         self.sources = sources
-        MultiOp.__init__(self, results)
+        MultiOp.__init__(self, results)#}}}
 
 
-class EquiJoinIndexOp(MultiMultiOp):
+class EquiJoinIndexOp(MultiMultiOp):#{{{
     __slots__ = ["jointype"]
 
-    def __init__(self, leftslice, rightslice, jointype="inner"):
-        leftslice = ensure_frozen(leftslice)
-        rightslice = ensure_frozen(rightslice)
-
-        leftslice = PackArrayOp(leftslice)
-        rightslice = PackArrayOp(rightslice)
-        
-        leftslice,rightslice= broadcast((leftslice,rightslice),mode="dim")[0]
-
+    def __init__(self, leftslice, rightslice, jointype="inner", mode="dim"):
         assert leftslice.dims == rightslice.dims, "Parent dims of join fields should be equal"
         name= leftslice.type.dims[0].name + "_" + rightslice.type.dims[0].name
         ndim = dimensions.Dim(UNDEFINED, (True,) * len(leftslice.dims), name=name)
@@ -691,7 +739,7 @@ class EquiJoinIndexOp(MultiMultiOp):
         r1 = SelectOp(self,0,leftslice.name, ltype,leftslice.dims, leftslice.bookmarks)
         r2 = SelectOp(self,1,rightslice.name, rtype,leftslice.dims, rightslice.bookmarks)
         self.jointype = jointype
-        MultiMultiOp.__init__(self, (leftslice,rightslice),(r1,r2))
+        MultiMultiOp.__init__(self, (leftslice,rightslice),(r1,r2))#}}}
 
 
 
