@@ -51,12 +51,11 @@ def yeast_aliases(feats, remove_multi=True):
     res = res/("alias", "feat_name")
     res = res%"feat_aliases"
     res = res[_.alias != ""].Unique()
-    
+    res = res.Each(str.upper, dtype="bytes")
     if remove_multi:
         multi_aliases = res.GroupBy(_.alias)[_.feat_name.Count() > 1].alias
         res = res[~(_.alias |In| multi_aliases)]
     
-    res = res.Each(str.upper, dtype="bytes")
     return res.Copy()
 predefined_sources.register(yeast_aliases,name="name_aliases",category="yeast")  
 
@@ -261,14 +260,61 @@ def go_annotations(dburl=config.get("databases.go_url",None), genus="Saccharomyc
     go = open_go(dburl)
     g = go.species |Match(_.id,                   _.species_id)|      go.gene_product
     g = g          |Match(_.gene_product.id,      _.gene_product_id)| go.association
-    g = g          |Match(_.association.term_id,  _.term2_id)|        go.graph_path
-    g = g          |Match(_.term1_id,             _.id)|              go.term//"annot"
-    g = g          |Match(_.relationship_type_id, _.id)|              go.term//"rel"
+    g = g          |Match(_.association.term_id,   _.id)|             go.term//"annot"
     g = g          |Match(_.association.id,       _.association_id)|  go.evidence
-    g = g[(_.genus==genus) & (_.species == species)][_.is_not == False]
-    return g.Get(_.symbol, _.annot.name/"annotation", _.rel.name/"relation_type", _.evidence.code, _.distance)%"annotations"
+    g = g          |Match(_.gene_product.dbxref_id,           _.id)|  go.dbxref
+    g = g[(_.genus==genus) & (_.species == species)][_.is_not == False][_.evidence.code != "ND"]
+    g = g.ReplaceMissing()
+    g = g.Get(_.symbol   /"gene_symbol",   _.xref_key / "gene_id",  
+                 _.annot.acc/"go_id",         _.annot.name/"annotation", _.annot.term_type/"go_type", 
+                 _.evidence.code/"evidence")%"annotations"
+    return g.Copy()
 predefined_sources.register(go_annotations,name="annotations",category="go")
 
+def go_info(dburl=config.get("databases.go_url",None), genus="Saccharomyces", species="cerevisiae"):
+    """Accesses GO term info in a MySQL database.
+
+       Database data can be obtained from the geneontology website.
+
+       example url: "mysql://username:password@hostname:port/go
+    """
+    go = open_go(dburl)
+    #select annotations from a certain species, and their ancestor annotations
+    g = go.species
+    g = g |Match(_.id,                  _.species_id)|      go.gene_product
+    g = g |Match(_.gene_product.id,     _.gene_product_id)| go.association
+    g = g |Match(_.association.term_id, _.term2_id)|        go.graph_path
+    g = g[(_.genus==genus) & (_.species == species)]
+
+    #link them to terms to get actual go accession id
+    g = g |Match(_.term2_id,             _.id)| go.term//"child"
+    g = g |Match(_.term1_id,             _.id)| go.term//"parent"
+    g = g |Match(_.relationship_type_id, _.id)| go.term//"rel"
+
+    #get relevant fields
+    g = g.ReplaceMissing()
+    g = g.Get(_.child.acc/"go_id", _.parent.acc/"parent_id", _.child.term_type/"go_type", _.child.name/"annotation",
+              _.distance, _.rel.name/"relationship", _.gene_product.id/"gene_id").Copy()
+
+    #step A: calculate number of genes associated with go terms
+    ngenes = g.GroupBy(_.parent_id).Get(_.parent_id/"go_id",_.gene_id.Unique().Count()/"ngenes")
+
+    #step B1: group per term
+    goinfo = g.GroupBy(_.go_id, flat=(_.go_type, _.annotation))
+    
+    #step B2: for ech term, determine type, max depth to root, ancestors (non-unique) and relationship to ancestors
+    goinfo = goinfo.Get(_.go_id, _.go_type, _.annotation,  
+                        _[_.parent_id == "all"].distance.Max()/"depth",
+                        _.parent_id / "ancestor", _.relationship, _.distance)
+
+    #step B3: select for each ancestor the minimum path length
+    goinfo = goinfo.GroupBy(_.ancestor)[..., _.distance.Argmin()].Copy()
+
+    #step C: combine, copy, return
+    return (goinfo |Match| ngenes).Copy()
+
+predefined_sources.register(go_info,name="term_info",category="go")
+
 
 ########################### #KEGG ######################################
 @util.memoized
