@@ -221,6 +221,13 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
     def visitChangeDimOp(self,node,slice):
         return slice.modify(rtype=node.type,dims=node.dims)
 
+    def visitDetectFixedShapesOp(self,node,slice):
+        data = slice.data.flat()
+        if len(data) == 0:
+            raise RuntimeError, "Cannot determine dim shape from empty data. Please cast dim shape of fixed dims."
+        ntype = detect_shape(data[0],slice.type)
+        return slice.modify(rtype=ntype)
+    
     def visitDetectTypeOp(self,node,slice):
         if(slice.type == rtypes.unknown):
             det = detector.Detector()
@@ -229,7 +236,7 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
             return slice.modify(rtype=det.getType())
         else:
             return slice
-    
+   
     def visitUnpackArrayOp(self,node,slice):
         ndata=slice.data.unpack(node.unpack_dims, subtype=node.type)
         return slice.modify(data=ndata,rtype=node.type,dims=node.dims)
@@ -268,6 +275,11 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
                 func = lambda x: x.get(didx,Missing)
             else:
                 func = operator.itemgetter(didx)
+        elif(isinstance(slice.type,rtypes.TypeIndexDict)):
+            if node.tuple_idx == 0:
+                func = lambda x: x.keys()
+            else:
+                func = lambda x: x.values()
         else:
             if(node.type.has_missing):
                 def func(x):
@@ -283,7 +295,19 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
     def visitPackTupleOp(self,node, slices):
         ndata = nested_array.co_mapseq(speedtuplify,[slice.data for slice in slices],res_type=node.type)
         return slices[0].modify(data=ndata,name=node.name,rtype=node.type,dims=node.dims,bookmarks=node.bookmarks)
-    
+   
+    def visitPackIndexDictOp(self, node, slices):
+        names = node.type.fieldnames
+        def speed_index_dictify(x):
+            return dict(zip(*x))
+        ndata = nested_array.co_map(speed_index_dictify,[slice.data for slice in slices],res_type=node.type)
+        return slices[0].modify(data=ndata,name=node.name,rtype=node.type,dims=node.dims,bookmarks=node.bookmarks)
+       
+    def visitTakeOp(self, node, slices):
+        ndata = nested_array.co_mapseq(speeddictindex,[slice.data for slice in slices],res_type=node.type, 
+                                        dtype=node.type.toNumpy(), allow_missing=node.allow_missing)
+        return slices[0].modify(data=ndata,name=node.name,rtype=node.type,dims=node.dims,bookmarks=node.bookmarks)
+
     def visitPackDictOp(self,node, slices):
         names = node.type.fieldnames
         if node.with_missing or not any([slice.type.has_missing for slice in slices]):
@@ -354,7 +378,7 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
                     rshape = add_independent(rshape,node.dims[pos])
                 repeat_dict[pos] = rshape
                 bcpos += 1
-            elif(planelem == BCCOPY):
+            elif(planelem == BCCOPY or planelem == BCSOURCE):
                 pass
             else:
                 raise RuntimeError, "Unknown broadcast plan element: " + str(planelem)
@@ -364,9 +388,9 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
 
     def visitFilterOp(self,node, slice, constraint):
         ndata = nested_array.co_map(speedfilter,[slice.data, constraint.data],
-                                       has_missing = node.has_missing,ctype=constraint.type,
+                                       has_missing = node.has_missing,ctype=constraint.type,stype=node.type,
                                        res_type=node.type)
-        return slice.modify(data=ndata,rtype=node.type)
+        return slice.modify(data=ndata,rtype=node.type, dims=node.dims)
 
     def visitFlatAllOp(self, node, slice):
         ndata = slice.data.mergeAllDims()
@@ -395,7 +419,7 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
         return slice.modify(data=ndata,rtype=node.type)
     
     def visitNoneToMissingOp(self, node, slice):
-        ndata = slice.data.mapseq(none_to_missing,res_type=node.type)
+        ndata = slice.data.mapseq(none_to_missing,res_type=node.type, stype=node.type)
         return slice.modify(data=ndata)
 
     def visitUnaryFuncSeqOp(self,node, slice):
@@ -469,7 +493,7 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
     def visitSelectOp(self, node, slice):
         return slice.modify(data=slice.data[node.index],name=node.name, rtype=node.type, dims=node.dims, bookmarks=node.bookmarks)
 
-    def visitFixateOp(self,node,slices):
+    def visitFixateOp(self,node,slices=[]):
         res = []
         for cur_slice, slice in zip(node.sources, slices):
             ndata = slice.data.getStructuredData()
@@ -499,7 +523,10 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
 
     def castto_numbers(self,castname,node,slice):
         dtype = node.type.toNumpy()
-        return slice.data.mapseq(lambda x:numpy.cast[dtype](x),res_type=node.type)
+        if not node.type.has_missing:
+            return slice.data.mapseq(lambda x:numpy.cast[dtype](x),res_type=node.type)
+        else:
+            raise RuntimeError, "Casting missing values not yet supported"
 
     def caststring_to_int(self, castname, node, slice):
         if(node.type.has_missing):
@@ -572,6 +599,17 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
                 raise RuntimeError, "Found unequal values during merge: " + str(lelem) + " != " + str(relem)
         return cutils.darray(res, typeo.toNumpy())
 
+    def simple_arithDivide(self, data, type1, type2, typeo, op):
+        data1,data2 = data
+        if(data1 is Missing or data2 is Missing):
+            return Missing
+        filter = (data2 == 0) & (data1 == 0)
+        if filter.any():
+            data2 = data2.copy()
+            data2[filter] = 1.0
+        res = numpy_arith[op](data1, data2, sig=typeo.toNumpy())
+        return res
+
     def simple_arithGeneral(self, data, type1, type2, typeo, op):
         data1,data2 = data
         if(data1 is Missing or data2 is Missing):
@@ -605,8 +643,15 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
         #such as "numpy.equal" on string arrays
         #so use direct operations ("__eq__")
         op = python_op[op]
+        #Note: numpy seems to segfault, when using reverse operations on string comparision
+        #using as first argument an  object array and as second a string array. 
+        if(isinstance(data[0],numpy.ndarray) and data[0].dtype == object and isinstance(data[1],numpy.ndarray) and data[1].dtype != object):
+            data = (data[0], numpy.cast[object](data[1]))
+
         res = getattr(data[0], op)(data[1])
         if(res is NotImplemented):
+            if(isinstance(data[0],numpy.ndarray) and data[0].dtype != object and isinstance(data[1],numpy.ndarray) and data[1].dtype == object):
+                data = (numpy.cast[object](data[0]), data[1])
             res = getattr(data[1], reverse_op[op])(data[0])
         assert not res is NotImplemented, "Not implemented error in stringstringGeneral for " \
                                             + str(op) + " and " + str(type1) + ", " + str(type2)
@@ -624,6 +669,30 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
                 res.append(not elem)
         return cutils.darray(res, type_out.toNumpy())
 
+    def repmissingReplaceMissing(self, data, type_in, type_out, op, def_value=NOVAL):
+        if not type_in.has_missing:
+            return data
+        if def_value is NOVAL:
+            if type_out.__class__ is rtypes.TypeArray and type_out.dims[0].shape == UNDEFINED:
+                for elem in data:
+                    if not elem is Missing:
+                        shape = len(elem)
+                        break
+                else:
+                    raise RuntimeError, "Could not determine shape of dimension. ReplaceMissing impossible. Change to variable dim."
+               
+                dim = dimensions.Dim(shape, type_out.dims[0].dependent)
+                def_value = dimpaths.dimsToArrays(dimpaths.DimPath(dim),type_out.subtypes[0]).toDefval()
+            else:                
+                def_value = type_out.toDefval()
+        res = []
+        for elem in data:
+            if elem is Missing:
+                res.append(def_value)
+            else:
+                res.append(elem)
+        return cutils.darray(res, type_out.toNumpy())
+    
     def corrCorr(self, data, type_in, type_out, op):
         intype = type_in.toNumpy()
         res = []
@@ -720,17 +789,42 @@ class PyExec(VisitorFactory(prefixes=("visit",), flags=NF_ELSE),
             return cutils.darray([[''.join(list(subrow)) for subrow in row.transpose()] for row in data],object,2,2)
         else:
             return cutils.darray([''.join(list(row)) for row in data],object)
+    
+    def fixdimArgmax(self, data, type_in, type_out, op, packdepth):
+        data = ensure_fixeddims(data,packdepth,type_in.toNumpy())
+        func = numpy_dimfuncs[op]
+        dtype = type_out.toNumpy()
+        if type_in.has_missing:               
+            res = []
+            for row in data:
+                pos = numpy.arange(len(row))
+                filter = ~numpy.equal(row, Missing)
+                pos = pos[filter]
+                row = row[filter]
+                res.append(pos[func(row,axis=0)])
+            return cutils.darray(res,dtype)
+        else:            
+            if(len(data.shape) < 2):
+                if(packdepth > 1):
+                    dtype = object
+                return cutils.darray([func(row,axis=0) for row in data],dtype)
+            else:
+                return func(data,axis=1)
+    fixdimArgmin = fixdimArgmax
 
     def fixdimGeneral(self, data, type_in, type_out, op, packdepth):
         data = ensure_fixeddims(data,packdepth,type_in.toNumpy())
         func = numpy_dimfuncs[op]
         dtype = type_out.toNumpy()
-        if(len(data.shape) < 2):
-            if(packdepth > 1):
-                dtype = object
-            return cutils.darray([func(row,axis=0) for row in data],dtype)
-        else:
-            return func(data,axis=1)
+        if type_in.has_missing:                
+            return cutils.darray([func([elem for elem in row if not elem is Missing],axis=0) for row in data],dtype)
+        else:            
+            if(len(data.shape) < 2):
+                if(packdepth > 1):
+                    dtype = object
+                return cutils.darray([func(row,axis=0) for row in data],dtype)
+            else:
+                return func(data,axis=1)
       
 numpy_cmp = {'Equal':numpy.equal,#{{{
             'NotEqual':numpy.not_equal,
@@ -759,7 +853,8 @@ numpy_dimfuncs = {
     'Sum':numpy.sum,
     'Median':numpy.median,
     'Argmin':numpy.argmin,
-    'Argmax':numpy.argmax
+    'Argmax':numpy.argmax,
+    'Std':numpy.std
     }
 
 python_op = {'Equal':'__eq__',
@@ -824,31 +919,42 @@ def speedarrayify(seqs,dtype):
     return nseq
 
 
-def speedfilter(seqs,has_missing, ctype):
+def speeddictindex(seqs,dtype, allow_missing=False):
+    if allow_missing:
+        return cutils.darray([d.get(k,Missing) for d, k in zip(*seqs)],dtype)
+    else:
+        return cutils.darray([d[k] for d, k in zip(*seqs)],dtype)
+
+
+def speedfilter(seqs,has_missing, ctype, stype):
     data,constraint = seqs
+    if data is Missing:
+        return data
+    if len(data) == 0:
+        return data
     if(has_missing):
         if(isinstance(ctype,rtypes.TypeArray)):
+            missing = stype.subtypes[0].toMissingval()
             if(isinstance(ctype.subtypes[0],rtypes.TypeBool)):
-                data = data.ravel()
                 res = []
                 for pos, elem in enumerate(constraint.ravel()):
                     if(elem is Missing):
-                        res.append(Missing)
+                        res.append(missing)
                     elif(elem == True):
                         res.append(data[pos])
                 res = cutils.darray(res,object)
             else:#indices
-                data = data.ravel()
                 res = []
                 for elem in constraint.ravel():
                     if(elem is Missing):
-                        res.append(Missing)
+                        res.append(missing)
                     else:
                         res.append(data[elem])
                 res = cutils.darray(res,object)
         else:
+            missing = stype.toMissingval()
             if(constraint is Missing):
-                res = Missing
+                res = missing
             else:
                 try:
                     res = data[constraint]
@@ -1044,9 +1150,20 @@ def string_to_real(seq, dtype):
     return cutils.darray([float(elem) for elem in seq],dtype)
 
 
-def none_to_missing(seq):
+def none_to_missing(seq,stype):
+    missing = stype.toMissingval()
     seq = seq.copy()
-    seq[numpy.equal(seq,None)] = Missing
+    seq[numpy.equal(seq,None)] = missing
     return seq
    
+def detect_shape(data, stype):
+    if stype.__class__ is rtypes.TypeArray:
+        if not stype.dims[0].dependent and stype.dims[0].shape == UNDEFINED:
+            stype.dims[0].shape = len(data)
+
+        subtype = detect_shape(data[0],stype.subtypes[0])
+        stype = stype.copy()
+        stype.subtypes= (subtype,)
+    
+    return stype
 
